@@ -1,37 +1,34 @@
 # encoding: utf-8
 
-'''
+"""
 .. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
-'''
+"""
 
 from __future__ import absolute_import, unicode_literals
 
-from .error import InvalidDataError
+from .error import DataError
 
 
-def to_value_matrix(header_list, value_matrix):
+def to_value_matrix(headers, value_matrix):
     if value_matrix is None:
         return []
 
-    return [
-        _to_row(header_list, value_list, row_idx)[1]
-        for row_idx, value_list in enumerate(value_matrix)
-    ]
+    return [_to_row(headers, values, row_idx)[1] for row_idx, values in enumerate(value_matrix)]
 
 
-def _to_row(header_list, values, row_idx):
-    if header_list:
+def _to_row(headers, values, row_idx):
+    if headers:
         try:
             values = values._asdict()
         except AttributeError:
             pass
 
         try:
-            return (row_idx, [values.get(header) for header in header_list])
+            return (row_idx, [values.get(header) for header in headers])
         except (TypeError, AttributeError):
             pass
 
     if not isinstance(values, (tuple, list)):
-        raise InvalidDataError("row must be a list or tuple: actual={}".format(values))
+        raise DataError("row must be a list or tuple: actual={}".format(type(values)))
 
     return (row_idx, values)
