@@ -7,8 +7,7 @@
 from __future__ import absolute_import, unicode_literals
 
 import re
-from collections import OrderedDict
-from decimal import Decimal
+from collections import OrderedDict, namedtuple
 
 import dataproperty as dp
 import six
@@ -19,7 +18,6 @@ from typepy import Nan
 from ._constant import PatternMatch
 from ._converter import to_value_matrix
 from ._logger import logger
-from .error import InvalidDataError
 
 
 class TableData(object):
@@ -27,8 +25,8 @@ class TableData(object):
     Class to represent a table data structure.
 
     :param str table_name: Name of the table.
-    :param list header_list: Table header names.
-    :param list record_list: Table data records.
+    :param list headers: Table header names.
+    :param list rows: Data of the table.
     """
 
     @property
@@ -45,36 +43,38 @@ class TableData(object):
         self.__table_name = value
 
     @property
-    def header_list(self):
-        """
-        :return: Table header names.
-        :rtype: list
+    def headers(self):
+        """Get the table header names.
+
+        Returns:
+            |list| or |tuple|: Table header names.
         """
 
-        return self.__dp_extractor.header_list
+        return self.__dp_extractor.headers
 
     @property
-    def row_list(self):
-        """
-        :return: Table rows.
-        :rtype: list
+    def rows(self):
+        """Original rows of tabular data.
+
+        Returns:
+            |list| or |tuple|: Table rows.
         """
 
-        return self.__row_list
+        return self.__rows
 
     @property
     def value_matrix(self):
-        """
-        :return: Table data rows.
-        :rtype: list
+        """Converted rows of tabular data.
+
+        Returns:
+            |list| or |tuple|: Table rows.
         """
 
         if self.__value_matrix:
             return self.__value_matrix
 
         self.__value_matrix = [
-            [value_dp.data for value_dp in value_dp_list]
-            for value_dp_list in self.value_dp_matrix
+            [value_dp.data for value_dp in value_dp_list] for value_dp_list in self.value_dp_matrix
         ]
 
         return self.__value_matrix
@@ -88,22 +88,22 @@ class TableData(object):
         """
         :return:
             Number of rows in the tabular data.
-            |None| if the ``row_list`` is neither list nor tuple.
+            |None| if the ``rows`` is neither list nor tuple.
         :rtype: int
         """
 
         try:
-            return len(self.row_list)
+            return len(self.rows)
         except TypeError:
             return None
 
     @property
     def num_columns(self):
-        if typepy.is_not_empty_sequence(self.header_list):
-            return len(self.header_list)
+        if typepy.is_not_empty_sequence(self.headers):
+            return len(self.headers)
 
         try:
-            return len(self.row_list[0])
+            return len(self.rows[0])
         except TypeError:
             return None
         except IndexError:
@@ -118,7 +118,8 @@ class TableData(object):
 
         if self.__value_dp_matrix is None:
             self.__value_dp_matrix = self.__dp_extractor.to_dp_matrix(
-                to_value_matrix(self.header_list, self.row_list))
+                to_value_matrix(self.headers, self.rows)
+            )
 
         return self.__value_dp_matrix
 
@@ -126,68 +127,73 @@ class TableData(object):
     def header_dp_list(self):
         return self.__dp_extractor.to_header_dp_list()
 
+    @property
+    def column_dp_list(self):
+        return self.__dp_extractor.to_column_dp_list(self.value_dp_matrix)
+
     @property
     def dp_extractor(self):
         return self.__dp_extractor
 
-    def __init__(self, table_name, header_list, row_list, dp_extractor=None):
+    def __init__(
+        self, table_name, headers, rows, dp_extractor=None, type_hints=None, max_workers=None
+    ):
         self.__table_name = table_name
         self.__value_matrix = None
         self.__value_dp_matrix = None
 
-        if row_list:
-            self.__row_list = row_list
+        if rows:
+            self.__rows = rows
         else:
-            self.__row_list = []
+            self.__rows = []
 
         if dp_extractor:
             self.__dp_extractor = dp_extractor
         else:
             self.__dp_extractor = dp.DataPropertyExtractor()
 
+        if type_hints:
+            self.__dp_extractor.column_type_hints = type_hints
+
         self.__dp_extractor.strip_str_header = '"'
 
         if six.PY2:
             # avoid unit test execution hang up at Python 2 environment
             self.__dp_extractor.max_workers = 1
-
-        if not header_list:
-            self.__dp_extractor.header_list = []
         else:
-            self.__dp_extractor.header_list = header_list
+            if max_workers:
+                self.__dp_extractor.max_workers = max_workers
+
+        if not headers:
+            self.__dp_extractor.headers = []
+        else:
+            self.__dp_extractor.headers = headers
 
     def __repr__(self):
-        element_list = [
-            "table_name={}".format(self.table_name),
-        ]
+        element_list = ["table_name={}".format(self.table_name)]
 
         try:
-            element_list.append("header_list=[{}]".format(", ".join(self.header_list)))
+            element_list.append("headers=[{}]".format(", ".join(self.headers)))
         except TypeError:
-            element_list.append("header_list=None")
+            element_list.append("headers=None")
 
-        element_list.append("rows={}".format(self.num_rows))
+        element_list.extend(["cols={}".format(self.num_columns), "rows={}".format(self.num_rows)])
 
         return ", ".join(element_list)
 
     def __eq__(self, other):
-        return self.equals(other, is_strict=True)
+        return self.equals(other, cmp_by_dp=True)
 
     def __ne__(self, other):
-        return not self.equals(other, is_strict=True)
+        return not self.equals(other, cmp_by_dp=True)
 
     def is_empty_header(self):
         """
-        :return: |True| if the data :py:attr:`.header_list` is empty.
+        :return: |True| if the data :py:attr:`.headers` is empty.
         :rtype: bool
         """
 
-        return typepy.is_empty_sequence(self.header_list)
-
-    def is_empty_record(self):
-        """Depricated"""
-
-        return self.is_empty_rows()
+        return typepy.is_empty_sequence(self.headers)
 
     def is_empty_rows(self):
         """
@@ -200,23 +206,21 @@ class TableData(object):
     def is_empty(self):
         """
         :return:
-            |True| if the data :py:attr:`.header_list` or
+            |True| if the data :py:attr:`.headers` or
             :py:attr:`.value_matrix` is empty.
         :rtype: bool
         """
 
         return any([self.is_empty_header(), self.is_empty_rows()])
 
-    def equals(self, other, is_strict=False):
-        if is_strict:
+    def equals(self, other, cmp_by_dp=False):
+        if cmp_by_dp:
             return self.__equals_raw(other)
 
         return self.__equals_dp(other)
 
     def __equals_base(self, other):
-        compare_item_list = [
-            self.table_name == other.table_name,
-        ]
+        compare_item_list = [self.table_name == other.table_name]
 
         if self.num_rows is not None:
             compare_item_list.append(self.num_rows == other.num_rows)
@@ -227,17 +231,20 @@ class TableData(object):
         if not self.__equals_base(other):
             return False
 
-        if self.header_list != other.header_list:
+        if self.headers != other.headers:
             return False
 
-        for lhs_row, rhs_row in zip(self.row_list, other.row_list):
+        for lhs_row, rhs_row in zip(self.rows, other.rows):
             if len(lhs_row) != len(rhs_row):
                 return False
 
-            if not all([
-                lhs == rhs for lhs, rhs in zip(lhs_row, rhs_row)
-                if not Nan(lhs).is_type() and not Nan(rhs).is_type()
-            ]):
+            if not all(
+                [
+                    lhs == rhs
+                    for lhs, rhs in zip(lhs_row, rhs_row)
+                    if not Nan(lhs).is_type() and not Nan(rhs).is_type()
+                ]
+            ):
                 return False
 
         return True
@@ -258,9 +265,9 @@ class TableData(object):
 
         return True
 
-    def in_tabledata_list(self, other, is_strict=False):
+    def in_tabledata_list(self, other, cmp_by_dp=False):
         for table_data in other:
-            if self.equals(table_data, is_strict=is_strict):
+            if self.equals(table_data, cmp_by_dp=cmp_by_dp):
                 return True
 
         return False
@@ -272,22 +279,29 @@ class TableData(object):
 
         invalid_row_idx_list = []
 
-        for row_idx, row in enumerate(self.row_list):
-            if isinstance(row, (list, tuple)) and len(self.header_list) != len(row):
+        for row_idx, row in enumerate(self.rows):
+            if isinstance(row, (list, tuple)) and len(self.headers) != len(row):
                 invalid_row_idx_list.append(row_idx)
 
             if isinstance(row, dict):
-                if not all([header in row for header in self.header_list]):
+                if not all([header in row for header in self.headers]):
                     invalid_row_idx_list.append(row_idx)
 
         if not invalid_row_idx_list:
             return
 
+        for invalid_row_idx in invalid_row_idx_list:
+            logger.debug(
+                "invalid row (line={}): {}".format(invalid_row_idx, self.rows[invalid_row_idx])
+            )
+
         raise ValueError(
-            "header length and value length are mismatch:\n" +
-            "  header({}): {}\n".format(len(self.header_list), self.header_list) +
-            "  # of miss match rows: {} ouf of {}\n".format(
-                len(invalid_row_idx_list), self.num_rows))
+            "table header length and row length are mismatch:\n"
+            + "  header(len={}): {}\n".format(len(self.headers), self.headers)
+            + "  # of miss match rows: {} ouf of {}\n".format(
+                len(invalid_row_idx_list), self.num_rows
+            )
+        )
 
     def as_dict(self):
         """
@@ -300,37 +314,67 @@ class TableData(object):
                 from tabledata import TableData
 
                 TableData(
-                    table_name="sample",
-                    header_list=["a", "b"],
-                    row_list=[[1, 2], [3.3, 4.4]]
+                    "sample",
+                    ["a", "b"],
+                    [[1, 2], [3.3, 4.4]]
                 ).as_dict()
 
         :Output:
             .. code:: json
 
-                {'sample': [OrderedDict([('a', 1), ('b', 2)]),
-                  OrderedDict([('a', 3.3), ('b', 4.4)])]}
+                {'sample': [OrderedDict([('a', 1), ('b', 2)]), OrderedDict([('a', 3.3), ('b', 4.4)])]}
         """
 
-        from typepy import Typecode
-
         dict_body = []
+        for row in self.value_matrix:
+            if not row:
+                continue
+
+            values = [
+                (header, value) for header, value in zip(self.headers, row) if value is not None
+            ]
+
+            if not values:
+                continue
+
+            dict_body.append(OrderedDict(values))
+
+        return {self.table_name: dict_body}
+
+    def as_tuple(self):
+        """
+        :return: Rows of the table.
+        :rtype: list of |namedtuple|
+
+        :Sample Code:
+            .. code:: python
+
+                from tabledata import TableData
+
+                records = TableData(
+                    "sample",
+                    ["a", "b"],
+                    [[1, 2], [3.3, 4.4]]
+                ).as_tuple()
+                for record in records:
+                    print(record)
+
+        :Output:
+            .. code-block:: none
+
+                Row(a=1, b=2)
+                Row(a=Decimal('3.3'), b=Decimal('4.4'))
+        """
+
+        Row = namedtuple("Row", self.headers)
+
         for value_dp_list in self.value_dp_matrix:
             if typepy.is_empty_sequence(value_dp_list):
                 continue
 
-            row = [
-                (header, value_dp.data)
-                for header, value_dp in zip(self.header_list, value_dp_list)
-                if value_dp.typecode != Typecode.NONE
-            ]
+            row = Row(*[value_dp.data for value_dp in value_dp_list])
 
-            if typepy.is_empty_sequence(row):
-                continue
-
-            dict_body.append(OrderedDict(row))
-
-        return {self.table_name: dict_body}
+            yield row
 
     def as_dataframe(self):
         """
@@ -343,10 +387,10 @@ class TableData(object):
                 from tabledata import TableData
 
                 TableData(
-                    table_name="sample",
-                    header_list=["a", "b"],
-                    row_list=[[1, 2], [3.3, 4.4]]
-                ).as_dict()
+                    "sample",
+                    ["a", "b"],
+                    [[1, 2], [3.3, 4.4]]
+                ).as_dataframe()
 
         :Output:
             .. code-block:: none
@@ -356,28 +400,32 @@ class TableData(object):
                 1  3.3  4.4
 
         :Dependency Packages:
-            - `pandas <http://pandas.pydata.org/>`__
+            - `pandas <https://pandas.pydata.org/>`__
         """
 
         import pandas
 
         dataframe = pandas.DataFrame(self.value_matrix)
         if not self.is_empty_header():
-            dataframe.columns = self.header_list
+            dataframe.columns = self.headers
 
         return dataframe
 
-    def filter_column(
-            self, pattern_list=None, is_invert_match=False,
-            is_re_match=False, pattern_match=PatternMatch.OR):
-        logger.debug(
-            "filter_column: pattern_list={}, is_invert_match={}, "
-            "is_re_match={}, pattern_match={}".format(
-                pattern_list, is_invert_match, is_re_match, pattern_match))
+    def transpose(self):
+        return TableData(self.table_name, self.headers, [row for row in zip(*self.rows)])
 
-        if not pattern_list:
-            return TableData(
-                table_name=self.table_name, header_list=self.header_list, row_list=self.row_list)
+    def filter_column(
+        self, patterns=None, is_invert_match=False, is_re_match=False, pattern_match=PatternMatch.OR
+    ):
+        logger.debug(
+            "filter_column: patterns={}, is_invert_match={}, "
+            "is_re_match={}, pattern_match={}".format(
+                patterns, is_invert_match, is_re_match, pattern_match
+            )
+        )
+
+        if not patterns:
+            return self
 
         match_header_list = []
         match_column_matrix = []
@@ -389,26 +437,26 @@ class TableData(object):
         else:
             raise ValueError("unknown matching: {}".format(pattern_match))
 
-        for header, column in zip(self.header_list, zip(*self.row_list)):
+        for header, column in zip(self.headers, zip(*self.rows)):
             is_match_list = []
-            for pattern in pattern_list:
+            for pattern in patterns:
                 is_match = self.__is_match(header, pattern, is_re_match)
 
-                is_match_list.append(any([
-                    is_match and not is_invert_match,
-                    not is_match and is_invert_match,
-                ]))
+                is_match_list.append(
+                    any([is_match and not is_invert_match, not is_match and is_invert_match])
+                )
 
             if match_method(is_match_list):
                 match_header_list.append(header)
                 match_column_matrix.append(column)
 
-        logger.debug("filter_column: table={}, match_header_list={}".format(
-            self.table_name, match_header_list))
+        logger.debug(
+            "filter_column: table={}, match_header_list={}".format(
+                self.table_name, match_header_list
+            )
+        )
 
-        return TableData(
-            table_name=self.table_name, header_list=match_header_list,
-            row_list=list(zip(*match_column_matrix)))
+        return TableData(self.table_name, match_header_list, list(zip(*match_column_matrix)))
 
     @staticmethod
     def from_dataframe(dataframe, table_name=""):
@@ -419,10 +467,7 @@ class TableData(object):
         :param str table_name: Table name to create.
         """
 
-        return TableData(
-            table_name=table_name,
-            header_list=list(dataframe.columns.values),
-            row_list=dataframe.values.tolist())
+        return TableData(table_name, list(dataframe.columns.values), dataframe.values.tolist())
 
     @staticmethod
     def __is_match(header, pattern, is_re_match):
@@ -430,78 +475,3 @@ class TableData(object):
             return re.search(pattern, header) is not None
 
         return header == pattern
-
-    def __to_value_matrix(self, value_matrix):
-        """
-        Convert matrix to rows
-        """
-
-        self.__dp_extractor.float_type = Decimal
-
-        if typepy.is_empty_sequence(self.header_list):
-            return value_matrix
-
-        if self.__dp_extractor.max_workers <= 1:
-            return self.__to_value_matrix_st(value_matrix)
-
-        return self.__to_value_matrix_mt(value_matrix)
-
-    def __to_value_matrix_st(self, value_matrix):
-        return [
-            _to_row_helper(self.__dp_extractor, self.header_list, value_list, row_idx)[1]
-            for row_idx, value_list in enumerate(value_matrix)
-        ]
-
-    def __to_value_matrix_mt(self, value_matrix):
-        from concurrent import futures
-
-        row_map = {}
-        try:
-            with futures.ProcessPoolExecutor(self.__dp_extractor.max_workers) as executor:
-                future_list = [
-                    executor.submit(
-                        _to_row_helper, self.__dp_extractor,
-                        self.header_list, value_list, row_idx)
-                    for row_idx, value_list in enumerate(value_matrix)
-                ]
-
-                for future in futures.as_completed(future_list):
-                    row_idx, row = future.result()
-                    row_map[row_idx] = row
-        finally:
-            logger.debug("shutdown ProcessPoolExecutor")
-            executor.shutdown()
-
-        return [row_map[row_idx] for row_idx in sorted(row_map)]
-
-
-def _to_row_helper(extractor, header_list, values, row_idx):
-    """
-    Convert values to a row.
-
-    :param values: Value to be converted.
-    :type values: |dict|/|namedtuple|/|list|/|tuple|
-    :raises ValueError: If the ``values`` is invalid.
-    """
-
-    try:
-        values = values._asdict()
-    except AttributeError:
-        pass
-
-    if isinstance(values, dict):
-        try:
-            return (
-                row_idx,
-                [
-                    dp.data
-                    for dp in extractor.to_dp_list([
-                        values.get(header) for header in header_list])
-                ])
-        except AttributeError:
-            pass
-
-    try:
-        return (row_idx, [dp.data for dp in extractor.to_dp_list(values)])
-    except TypeError:
-        raise InvalidDataError("row must be a list or tuple: actual={}".format(values))
